import * as i0 from '@angular/core';
import { NgModule, Injectable } from '@angular/core';
import { CommonModule } from '@angular/common';
import { __awaiter } from 'tslib';
import jwt_decode from 'jwt-decode';
import { lastValueFrom, of } from 'rxjs';
import * as i1 from '@angular/common/http';
import { HttpHeaders, HttpParams } from '@angular/common/http';

class CoreModule {
}
CoreModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: CoreModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CoreModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.7", ngImport: i0, type: CoreModule, imports: [CommonModule] });
CoreModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: CoreModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: CoreModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [CommonModule],
                }]
        }] });

class KeycloakAngularAuthorizationModule {
}
KeycloakAngularAuthorizationModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAngularAuthorizationModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
KeycloakAngularAuthorizationModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAngularAuthorizationModule, imports: [CoreModule], exports: [CoreModule] });
KeycloakAngularAuthorizationModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAngularAuthorizationModule, imports: [CoreModule, CoreModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAngularAuthorizationModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CoreModule],
                    exports: [CoreModule],
                }]
        }] });

class KeycloakAuthorizationGuard {
    constructor(router, keycloakAuthentication, keycloakAuthorization) {
        this.router = router;
        this.keycloakAuthentication = keycloakAuthentication;
        this.keycloakAuthorization = keycloakAuthorization;
    }
    canActivate(route, state) {
        return __awaiter(this, void 0, void 0, function* () {
            this.authenticated = yield this.keycloakAuthentication.isLoggedIn();
            this.permissions = this.keycloakAuthorization.permissions;
            // Force the user to log in if currently unauthenticated.
            if (!this.authenticated) {
                yield this.keycloakAuthentication.login({
                    redirectUri: window.location.origin + state.url,
                });
            }
            // Request permissions from the UMA2 endpoints at least once,
            // if a default resource server was specified at init time.
            if (this.permissions == null &&
                this.keycloakAuthorization.defaultResourceServerId != null) {
                yield this.keycloakAuthorization.obtainEntitlements();
            }
            this.permissions = this.keycloakAuthorization.permissions;
            const result = yield this.isAccessAllowed(route, state);
            return result;
        });
    }
}

/**
 * An Angular service inspired by the `keycloak-angular` library that extends
 * the functionality to include support for the SharedAuthorization features of Keycloak.
 *
 * This class should be initalized at bootstrap via the {@link init} method.
 */
class KeycloakAuthorizationService {
    constructor(http) {
        this.http = http;
    }
    /** see {@link KeycloakAuthorizationOptions.defaultResourceServerId }. */
    get defaultResourceServerId() {
        return this._defaultResourceServerId;
    }
    /**
     * The list of permissions of the current user contained in the
     * RPT token retrieved from the entitlment endpoint.
     *
     * `Undefined`, if the entitlment endpoint was never inquired
     * (see {@link obtainEntitlements}).
     */
    get permissions() {
        return this._permissions != null ? [...this._permissions] : undefined;
    }
    /**
     * Handles the initialization of the `keycloak-angular-authorization` library.
     * It must be called before invoking any operation of the library.
     *
     * @param options The initialization options for this library
     * (see {@link KeycloakAuthorizationOptions}).
     */
    init(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const keycloakConfig$ = this.getKeycloakConfig(options.config);
            this._keycloakConfig = yield lastValueFrom(keycloakConfig$);
            this._defaultResourceServerId = options.defaultResourceServerId;
            const umaConfig$ = this.inspectUmaConfigurationEndpoint();
            this._umaConfig = yield lastValueFrom(umaConfig$);
        });
    }
    /**
     * Obtains the entitlments for the current user for the resource server via Keycloak,
     * extracts and persists the contained permissions, then saves the obtained RPT for future reuse.
     *
     * @param resourceServerId The id of the resource server for which the entitlements are checked.
     * If undefined, the default {@link KeycloakAuthorizationOptions.defaultResourceServerId}
     * will be used (throwing an error when no default was set either).
     */
    obtainEntitlements(resourceServerId) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const res$ = this.obtainEntitlementsFromEndpoint(resourceServerId);
            const res = yield lastValueFrom(res$);
            if (res.access_token) {
                const decodedToken = jwt_decode(res.access_token);
                const permissions = (_b = (_a = decodedToken === null || decodedToken === void 0 ? void 0 : decodedToken.authorization) === null || _a === void 0 ? void 0 : _a.permissions) !== null && _b !== void 0 ? _b : [];
                this._rptRequestResult = res;
                this._permissions = permissions;
            }
        });
    }
    /**
     * Check whether the user has a permission to access a specific resource/scope.
     *
     * @param permissionToCheck The permission ({@link KeycloakPermission}) for which
     * to check authorization against the current user permissions.
     * @returns The {@link AssignedKeycloakPermission} if found, otherwhise `undefined`.
     */
    checkPermission(permissionToCheck) {
        var _a;
        const scopedAssignedPermission = (_a = this._permissions) === null || _a === void 0 ? void 0 : _a.find((permission) => {
            var _a;
            return (permissionToCheck.rsname == permission.rsname &&
                !permissionToCheck.scope) ||
                ((_a = permission.scopes) === null || _a === void 0 ? void 0 : _a.includes(permissionToCheck.scope));
        });
        return scopedAssignedPermission;
    }
    /**
     * Checks whether the user is authorized to access a specific resource/scope.
     *
     * @param permissionToCheck The permission ({@link KeycloakPermission}) for which
     * to check authorization against the current user permissions.
     * @returns `True` if the user is authorized, otherwhise `false`.
     */
    checkAuthorization(permissionToCheck) {
        const retrievedPermission = this.checkPermission(permissionToCheck);
        return !!retrievedPermission;
    }
    /**
     * Returns the {@link Keycloak.KeycloakConfig} object,
     * possibily loaded from a configuration file.
     *
     * @param config See {@link KeycloakAuthorizationOptions.config}.
     * @returns An {@link Observable} of the Keycloak configuration
     * ({@link eycloak.KeycloakConfig}).
     */
    getKeycloakConfig(config) {
        if (typeof config == 'object') {
            return of(config);
        }
        else {
            const configUrl = typeof config == 'string' ? config : 'keycloak.json';
            return this.http.get(configUrl, {});
        }
    }
    /**
     * Calls the endpoint for the discovery document including UMA 2.0 compliant endpoint
     * locations and capabilities exposed by Keycloak.
     *
     * @returns An {@link Observable} of the result of the call containing the
     * discovery document ({@link UmaConfigurationRequestResult}).
     */
    inspectUmaConfigurationEndpoint() {
        var _a, _b;
        if (!((_a = this._keycloakConfig) === null || _a === void 0 ? void 0 : _a.url))
            throw new Error('url');
        if (!((_b = this._keycloakConfig) === null || _b === void 0 ? void 0 : _b.realm))
            throw new Error('realm');
        const endpoint = this._keycloakConfig.url +
            '/realms/' +
            this._keycloakConfig.realm +
            '/.well-known/uma2-configuration';
        return this.http.get(endpoint);
    }
    /**
     * Obtains the entitlments for the current user for the resource server via Keycloak.
     *
     * @param resourceServerId The id of the resource server for which the entitlements are checked.
     * If undefined, the default {@link KeycloakAuthorizationOptions.defaultResourceServerId}
     * will be used (throwing an error when no default was set either).
     * @returns An {@link Observable} of the result of the call containing
     * the RPT ({@link RptRequestResult.access_token}).
     */
    obtainEntitlementsFromEndpoint(resourceServerId) {
        var _a;
        resourceServerId !== null && resourceServerId !== void 0 ? resourceServerId : (resourceServerId = this._defaultResourceServerId);
        if (!resourceServerId)
            throw new Error('resourceServerId');
        if (!((_a = this._keycloakConfig) === null || _a === void 0 ? void 0 : _a.clientId))
            throw new Error('clientId');
        if (!this._umaConfig)
            throw new Error('umaConfig');
        const headers = new HttpHeaders({
            'Content-type': 'application/x-www-form-urlencoded',
        });
        const params = new HttpParams({
            fromObject: {
                grant_type: 'urn:ietf:params:oauth:grant-type:uma-ticket',
                client_id: this._keycloakConfig.clientId,
                audience: resourceServerId,
            },
        });
        if (this._rptRequestResult) {
            params.set('rpt', this._rptRequestResult.access_token);
        }
        return this.http.post(this._umaConfig.token_endpoint, params.toString(), {
            headers: headers,
            responseType: 'json',
        });
    }
}
KeycloakAuthorizationService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAuthorizationService, deps: [{ token: i1.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
KeycloakAuthorizationService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAuthorizationService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: KeycloakAuthorizationService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { CoreModule, KeycloakAngularAuthorizationModule, KeycloakAuthorizationGuard, KeycloakAuthorizationService };
//# sourceMappingURL=caritas-keycloak-angular-authorization.mjs.map
