import { HttpClient } from '@angular/common/http';
import type { AssignedKeycloakPermission, KeycloakAuthorizationOptions, KeycloakPermission } from '../interfaces';
import * as i0 from "@angular/core";
/** @private Undocumented. */
export interface Rpt {
    authorization: {
        permissions: AssignedKeycloakPermission[];
    };
}
/**
 * An Angular service inspired by the `keycloak-angular` library that extends
 * the functionality to include support for the SharedAuthorization features of Keycloak.
 *
 * This class should be initalized at bootstrap via the {@link init} method.
 */
export declare class KeycloakAuthorizationService {
    private readonly http;
    /** @private See {@link KeycloakAuthorizationOptions.config }. */
    private _keycloakConfig?;
    /** @private See {@link KeycloakAuthorizationOptions.defaultResourceServerId }. */
    private _defaultResourceServerId?;
    /** @private Undocumented. */
    private _umaConfig?;
    /** @private Undocumented. */
    private _rptRequestResult?;
    /** @private See {@link permissions}. */
    private _permissions?;
    constructor(http: HttpClient);
    /** see {@link KeycloakAuthorizationOptions.defaultResourceServerId }. */
    get defaultResourceServerId(): string | undefined;
    /**
     * The list of permissions of the current user contained in the
     * RPT token retrieved from the entitlment endpoint.
     *
     * `Undefined`, if the entitlment endpoint was never inquired
     * (see {@link obtainEntitlements}).
     */
    get permissions(): AssignedKeycloakPermission[] | undefined;
    /**
     * Handles the initialization of the `keycloak-angular-authorization` library.
     * It must be called before invoking any operation of the library.
     *
     * @param options The initialization options for this library
     * (see {@link KeycloakAuthorizationOptions}).
     */
    init(options: KeycloakAuthorizationOptions): Promise<void>;
    /**
     * Obtains the entitlments for the current user for the resource server via Keycloak,
     * extracts and persists the contained permissions, then saves the obtained RPT for future reuse.
     *
     * @param resourceServerId The id of the resource server for which the entitlements are checked.
     * If undefined, the default {@link KeycloakAuthorizationOptions.defaultResourceServerId}
     * will be used (throwing an error when no default was set either).
     */
    obtainEntitlements(resourceServerId?: string): Promise<void>;
    /**
     * Check whether the user has a permission to access a specific resource/scope.
     *
     * @param permissionToCheck The permission ({@link KeycloakPermission}) for which
     * to check authorization against the current user permissions.
     * @returns The {@link AssignedKeycloakPermission} if found, otherwhise `undefined`.
     */
    checkPermission(permissionToCheck: KeycloakPermission): AssignedKeycloakPermission | undefined;
    /**
     * Checks whether the user is authorized to access a specific resource/scope.
     *
     * @param permissionToCheck The permission ({@link KeycloakPermission}) for which
     * to check authorization against the current user permissions.
     * @returns `True` if the user is authorized, otherwhise `false`.
     */
    checkAuthorization(permissionToCheck: KeycloakPermission): boolean;
    /**
     * Returns the {@link Keycloak.KeycloakConfig} object,
     * possibily loaded from a configuration file.
     *
     * @param config See {@link KeycloakAuthorizationOptions.config}.
     * @returns An {@link Observable} of the Keycloak configuration
     * ({@link eycloak.KeycloakConfig}).
     */
    private getKeycloakConfig;
    /**
     * Calls the endpoint for the discovery document including UMA 2.0 compliant endpoint
     * locations and capabilities exposed by Keycloak.
     *
     * @returns An {@link Observable} of the result of the call containing the
     * discovery document ({@link UmaConfigurationRequestResult}).
     */
    private inspectUmaConfigurationEndpoint;
    /**
     * Obtains the entitlments for the current user for the resource server via Keycloak.
     *
     * @param resourceServerId The id of the resource server for which the entitlements are checked.
     * If undefined, the default {@link KeycloakAuthorizationOptions.defaultResourceServerId}
     * will be used (throwing an error when no default was set either).
     * @returns An {@link Observable} of the result of the call containing
     * the RPT ({@link RptRequestResult.access_token}).
     */
    private obtainEntitlementsFromEndpoint;
    static ɵfac: i0.ɵɵFactoryDeclaration<KeycloakAuthorizationService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<KeycloakAuthorizationService>;
}
